package de.fraunhofer.iem.dataprovider.metrics

import de.fraunhofer.iem.dataprovider.gitlab.Platform
import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.toolRun.ToolRunService
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

data class ToolResultsDto(val repo: RepoDto, val toolRuns: List<ToolRunDto>)
data class RepoDto(val repoName: String, val repoUrl: String)
data class ToolDto(val name: String, val version: String)
data class ToolRunDto(val findings: List<FindingDto>, val tool: ToolDto)
data class FindingDto(val message: String)

@RestController
@RequestMapping("/metrics")
class MetricsController(private val toolRunService: ToolRunService) {

    private val logger = getLogger(javaClass)

    @GetMapping("/latestRuns/{platform}/{id}")
    suspend fun repoChanged(@PathVariable platform: Platform, @PathVariable id: Long): ToolResultsDto {
        logger.info("Get latest tool runs for repo $id on platform $platform received.")
        val toolResults = toolRunService.getLatestToolRunsForRepo(id, platform)
        val repo = toolResults.first().repository!!
        val repoDto = RepoDto(repo.name!!, repo.url!!)
        val trDto = toolResults.map {
            ToolRunDto(
                it.findings.map { finding -> FindingDto(finding.message ?: "NO FINDING MESSAGE.") },
                ToolDto(it.tool?.name!!, it.tool?.version!!)
            )
        }
        return ToolResultsDto(repoDto, trDto)

    }
}